.pdf-outer-container{align-items:center;display:flex;height:100%;justify-content:center;overflow:hidden;padding-left:16px;width:100%}.pdf-viewer{display:flex;flex-direction:column;height:100%;overflow-y:scroll}.paginator{background-color:hsla(0,0%,98%,.6);padding-bottom:15px;padding-top:10px;position:sticky;top:0;z-index:1001}.document,.paginator{align-items:center;display:flex;justify-content:center}
/*# sourceMappingURL=751.335db1f6.chunk.css.map*/